/* db.c */
#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>
#include <bc.h>
#include <glib.h>
#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixdata.h>

#include "db.h"
#include "libpq-fe.h"
#include "popup.h"

#ifdef WIN32
#define SOCKVERSION (0x0202)	/* version of winsock to request - I think 2.2 is apropriate */
#include <winsock2.h>
#include <windows.h>
#define sleep(x) Sleep( 1000 *(x))
#define usleep(x) Sleep((x)/1000)
#else
#define SOCKET int
#define INVALID_SOCKET (-1)
#endif

/* this function is FOUND in tga.h (don't ask!) */
int LoadTGATexture(Texture * texture, char *filename);

extern int OPERATING_MODE;

//int cardno=0; //// ERASE!! ////
extern FILE *onscreenpipes[];

static PGconn *conn = NULL;

//static PQnoticeReceiver oldreceiver=NULL;

/*
void newreceiver(void *arg, const PGresult *res)
{
  
 printf(stderr,"RECEIVER:arg=%s\n",(char*)arg);
 printf(stderr,"resstat :%s\n",PQresStatus(PQresultStatus(res)));
 printf(stderr,"msg     :%s\n",PQresultErrorMessage(res)); 
  exit();
}
*/

char* mystrcat( char* dest, char* src )
{
                while (*dest) dest++;

                while ((*dest++ = *src++));
                return --dest;
}

char* setMyConnInfo( char *pghost, char *pgport, char *pgoptions, char *pgtty, char *username, char *password, char *dbname, char *pgtimeout, char * sslmode )
{
        char* bigString;
        char blank[2] ="";
        char *endline = " \n";
        char *eqlhost ="host=";
        char *eqlport ="port=";
        char *eqldbname ="dbname=";
        char *eqluser ="user=";
        char *eqlpassword ="password=";
        char *eqltimout ="connect_timeout=";
        char * eqlsslmode ="sslmode=";

        int i = 0;

        if (pghost)
                i += strlen("host=")
                        + strlen(pghost)
                        + strlen(endline);
        if(pgport)
                i += strlen("port=")
                        + strlen(pgport)
                        + strlen(endline);
        if(dbname)
                i += strlen("dbname=")
                        + strlen(dbname)
                        + strlen(endline);
        if(username)
                i += strlen("user=")
                        + strlen(username)
                        + strlen(endline);

        if(password)
                i += strlen("password=")
                        + strlen(password)
                        + strlen(endline);

        if(pgtimeout)
                i += strlen("connect_timeout=")
                        + strlen(pgtimeout)
                        + strlen(endline);
        if (sslmode)
                i += strlen("sslmode=") + strlen(sslmode) + strlen(endline);

        if (pgoptions==NULL)
                pgoptions = blank;
        if (pgtty ==NULL)
                pgtty = blank; //else do somthing
        bigString = (char*) mem_alloc(i + 1);
        /* remember space for null terminator! */
        char *p = bigString;
        bigString[0] = '\0';
        if(pghost)
        {
                p = mystrcat(p,eqlhost);
                p = mystrcat(p,pghost);
                p = mystrcat(p,endline);
        }
        if(pgport)
        {
                p = mystrcat(p,eqlport);
                p = mystrcat(p,pgport);
                p = mystrcat(p,endline);
        }
        if(dbname)
        {
                p = mystrcat(p,eqldbname);
                p = mystrcat(p,dbname);
                p = mystrcat(p,endline);
        }
        if(username)
        {
                p = mystrcat(p,eqluser);
                p = mystrcat(p,username);
                p = mystrcat(p,endline);
        }
        if(password)
        {
                p = mystrcat(p,eqlpassword);
                p = mystrcat(p,password);
                p = mystrcat(p,endline);
        }
        if(eqltimout)
        {
                p = mystrcat(p,eqltimout);
                p = mystrcat(p,pgtimeout);
                p = mystrcat(p,endline);
        }
        if (sslmode)
        {
                p = mystrcat(p, eqlsslmode);
                p = mystrcat(p, sslmode);
                p = mystrcat(p,endline);
        }
        return (char *)bigString;
}


static int freshdb;
static int ConnectDB(void)
{
	/* set up database related stuff */
	char *pghost, *pgport, *pgoptions, *pgtty, *username, *password, *dbname;
	PGresult *res;
	char buffer[1024];
// FIXME -- remove database_ config keys after shell.conf rollout.
	dbname = bcconfig_getvalue("db_name") ;
	pghost = bcconfig_getvalue("db_host") ;
	pgport = bcconfig_getvalue("db_port") ;
	pgoptions = bcconfig_getvalue("db_opts") ? : bcconfig_getvalue("database_opts");
	pgtty = NULL;
	username = bcconfig_getvalue("db_username") ;
	password = bcconfig_getvalue("db_password") ;

	/* make a connection to the database */
	//conn = PQsetdbLogin(pghost, pgport, pgoptions, pgtty, dbname, username, password);
	conn = PQconnectStart( setMyConnInfo( pghost, pgport, pgoptions, pgtty, username, password, dbname, "10", "allow" ) );

	/* check to see that the backend connection was successfully made */
	/*if (PQstatus(conn) == CONNECTION_BAD)
	{
		fprintf(stderr, "Connection to database '%s' failed: %s\n", dbname, PQerrorMessage(conn));
		return 1;
	}*/
	PostgresPollingStatusType syncstatus;
	do {
		syncstatus = PQconnectPoll(conn);
		if (syncstatus == PGRES_POLLING_OK || syncstatus == PGRES_POLLING_FAILED )
			break;
		usleep(100000);
	} while (1);
	
	printf("\n aSynchronus Connection Status: %s\n", syncstatus == PGRES_POLLING_OK ? "PGRES_POLLING_OK" : syncstatus == PGRES_POLLING_FAILED ? "PGRES_POLLING_FAILED" : "unknown!" );	
			

	sprintf(buffer, "LISTEN %s", bcconfig_getvalue("popup_listen") ? : "doormsg");
	res = PQexec(conn, buffer);
	if (PQresultStatus(res) != PGRES_COMMAND_OK)
	{
		fprintf(stderr, "LISTEN command failed: %s", PQerrorMessage(conn));
		PQclear(res);
		disConnectDB();
		return 1;
	}
	res = PQexec(conn, "LISTEN shutdown");
	if (PQresultStatus(res) != PGRES_COMMAND_OK)
	{
		fprintf(stderr, "LISTEN command failed: %s", PQerrorMessage(conn));
		PQclear(res);
		disConnectDB();
		return 1;
	}
	if (res)
		PQclear(res);
	freshdb = 1;
	return 0;

}

static int reConnectDB()
{
	int retries = 10;
	int r = 0;
	while (retries--)
	{
		if (conn)
			PQfinish(conn);
		if (!(r = ConnectDB()))
			break;
		sleep(1);
	}
	return r;
}

void disConnectDB(void)
{
	if (conn)
		PQfinish(conn);
	conn = NULL;
}

static int lastid = 0;		/* last ID seen in a result row */

static int updatelast(PGresult * res)
{
	PopupWindow *win = NULL;
	extern GList *winlist;
	GError *error = NULL;
	char *message = NULL, *pimage = NULL, *message2 = NULL, *alerts = NULL;
	long colour, colour2;	/* last ID seen in a result row */
	int rows, n = 0, id = 0;

	rows = PQntuples(res);
	printf("got %d rows\n", rows);
	if (rows <= 0)
	{
		errormsg("No rows present, swipe a card");
		return 0;
	}

	if (lastid)
	{
		for (n = 0; n < rows; n++)
		{
			if (ignoredoor(atoi(PQgetvalue(res, n, 1))))
			{
				printf ("Door set to be ignored\n");
				continue;
			}
			Texture *tex = NULL;
			win = mem_calloc(sizeof(PopupWindow));
			winlist = g_list_append(winlist, win);
			printf("Opperating Mode %d\n.", OPERATING_MODE);
			if (OPERATING_MODE == 1)
			{
				//no text, just an image at original size
				pimage = PQgetvalue(res, n, 9);
				id = atoi(PQgetvalue(res, n, 0));
				printf("Sound effect is called next.\n");
				soundeffect(*(PQgetvalue(res, n, 13)));
				if (pimage && *pimage)
				{
				int span = strspn(pimage, "0123456789");
					if (span && pimage[span] == '|' && atoi(pimage) == 0)
					{	// It's an old image from the database 
						tex = mem_calloc(sizeof(Texture));
						gsize dlen;
						guint8 *stream = (guint8 *) g_base64_decode(pimage + span + 1, &dlen);
						GdkPixbuf *buf, *buf3;
						GdkPixbuf *buf2 =
						    gdk_pixbuf_new_from_inline(dlen, stream, TRUE, &error);

						if (error)
							errormsg("An error occurred: %s\n", error->message);
						g_free(stream);
						tex->truewidth = gdk_pixbuf_get_width(buf2);
						tex->trueheight = gdk_pixbuf_get_height(buf2);
						tex->aspectratio = ((float)tex->truewidth) / ((float)tex->trueheight);

						//buf=sizefix(buf2,256,256);
						buf3 = gdk_pixbuf_scale_simple(buf2, 256, 256, GDK_INTERP_TILES);
						g_assert(buf3);
						buf = gdk_pixbuf_flip(buf3, FALSE);
						tex->width = 256;
						tex->height = 256;
						tex->truewidth = gdk_pixbuf_get_width(buf2);
						tex->trueheight = gdk_pixbuf_get_height(buf2);
						tex->channels = (GLuint) gdk_pixbuf_get_n_channels(buf);
						tex->bpp = (GLuint) gdk_pixbuf_get_bits_per_sample(buf) * tex->channels;

						printf("channels: %d, %d\n", gdk_pixbuf_get_n_channels(buf),
						       gdk_pixbuf_get_bits_per_sample(buf));

						tex->imageData = (GLubyte *) gdk_pixbuf_get_pixels(buf);

						createWindow(win, tex, "", 0, "", 0, "", FALSE);
						g_object_unref(buf3);
						g_object_unref(buf2);
						g_object_unref(buf);
					} else if (span && pimage[span] == '|' && atoi(pimage) == 1)
					{	// It's a jpeg image from the database 
						tex = mem_calloc(sizeof(Texture));
						gsize dlen;
						guint8 *stream;
						GdkPixbuf *buf, *buf3;
						gboolean success;
						GdkPixbufLoader *loader =
						    gdk_pixbuf_loader_new_with_type("jpeg", &error);
						if (error)
							errormsg("An error occurred: %s\n", error->message);

						printf("\n\n\n!!! TARGET \n\n\n");
						fflush(NULL);
						exit(99);

						stream = (guint8 *) g_base64_decode(pimage + span + 1, &dlen);
						success = gdk_pixbuf_loader_write(loader, stream, dlen, &error);
						g_free(stream);	/* CHECKED: this should be g _ f r e e */
						if (success)
							success = gdk_pixbuf_loader_close(loader, &error);
						if (!success)
						{
							if (error)
							{
								errormsg("An error occurred: %s\n", error->message);
							} else
							{
								errormsg("An unknown error occurred.");
							}
							g_object_unref(loader);
							return -1;
						}

						GdkPixbuf *buf2 = gdk_pixbuf_loader_get_pixbuf(loader);
						g_object_ref(buf2); //should be un-necessary, buf2 is already referenced, and will increase ref count one more time than expected
						g_object_unref(loader);

						tex->truewidth = gdk_pixbuf_get_width(buf2);
						tex->trueheight = gdk_pixbuf_get_height(buf2);
						tex->aspectratio = ((float)tex->truewidth) / ((float)tex->trueheight);

						buf3 = gdk_pixbuf_scale_simple(buf2, 256, 256, GDK_INTERP_TILES);
						g_assert(buf3);
						buf = gdk_pixbuf_flip(buf3, FALSE);
						tex->width = 256;
						tex->height = 256;
						tex->truewidth = gdk_pixbuf_get_width(buf2);
						tex->trueheight = gdk_pixbuf_get_height(buf2);
						tex->channels = (GLuint) gdk_pixbuf_get_n_channels(buf);
						tex->bpp = (GLuint) gdk_pixbuf_get_bits_per_sample(buf) * tex->channels;

						printf("channels: %d, %d\n", gdk_pixbuf_get_n_channels(buf),
						       gdk_pixbuf_get_bits_per_sample(buf));

						tex->imageData = (GLubyte *) gdk_pixbuf_get_pixels(buf);

						createWindow(win, tex, "", 0, "", 0, "", FALSE);
						g_object_unref(buf3);
						buf3 = NULL;
						g_object_unref(buf2);
						buf2=NULL;
						g_object_unref(buf);
						buf=NULL;
					} else
					{
						tex = mem_calloc(sizeof(Texture));
						if (LoadTGATexture(tex, pimage) == 0)
						{
							createWindow(win, tex, "", 0, "", 0, "", FALSE);
						} else
						{
							mem_free(tex);
							errormsg
							    ("An error occurred: Invalid image stored for the doormsg\n");;
						}
					}
				} else
				{
					pimage = PQgetvalue(res, n, 9);
					if (pimage && *pimage)
					{
						int span = strspn(pimage, "0123456789");
						if (span && pimage[span] == '|' && atoi(pimage) == 0)
						{	// It's an image from the database 
							tex = mem_calloc(sizeof(Texture));
							gsize dlen;
							guint8 *stream =
							    (guint8 *) g_base64_decode(pimage + span + 1, &dlen);
							GdkPixbuf *buf, *buf3;
							GdkPixbuf *buf2 =
							    gdk_pixbuf_new_from_inline(dlen, stream, TRUE, &error);

							if (error)
								errormsg("An error occurred: %s\n", error->message);
							g_free(stream);
							tex->truewidth = gdk_pixbuf_get_width(buf2);
							tex->trueheight = gdk_pixbuf_get_height(buf2);
							tex->aspectratio =
							    ((float)tex->truewidth) / ((float)tex->trueheight);

							//buf=sizefix(buf2,256,256);
							buf3 =
							    gdk_pixbuf_scale_simple(buf2, 256, 256, GDK_INTERP_TILES);
							    g_assert(buf3);
							buf = gdk_pixbuf_flip(buf3, FALSE);
							tex->width = 256;
							tex->height = 256;
							tex->channels = (GLuint) gdk_pixbuf_get_n_channels(buf);
							tex->bpp =
							    (GLuint) gdk_pixbuf_get_bits_per_sample(buf) *
							    tex->channels;

							printf("channels: %d, %d\n", gdk_pixbuf_get_n_channels(buf),
							       gdk_pixbuf_get_bits_per_sample(buf));

							tex->imageData = (GLubyte *) gdk_pixbuf_get_pixels(buf);

							createWindow(win, tex, "", 0, "", 0, "", FALSE);
							g_object_unref(buf3);
							g_object_unref(buf2);
							g_object_unref(buf);
						} else if (span && pimage[span] == '|' && atoi(pimage) == 1)
						{	// It's a jpeg image from the database 
							tex = mem_calloc(sizeof(Texture));
							gsize dlen;
							guint8 *stream;
							GdkPixbuf *buf, *buf3;
							gboolean success;
							GdkPixbufLoader *loader =
							    gdk_pixbuf_loader_new_with_type("jpeg", &error);
							if (error)
								errormsg("An error occurred: %s\n", error->message);

							stream = (guint8 *) g_base64_decode(pimage + span + 1, &dlen);
							success = gdk_pixbuf_loader_write(loader, stream, dlen, &error);
							g_free(stream);	/* CHECKED: this should be g _ f r e e */
							if (success)
								success = gdk_pixbuf_loader_close(loader, &error);
							if (!success)
							{
								if (error)
								{
									errormsg("An error occurred: %s\n",
										 error->message);
								} else
								{
									errormsg("An unknown error occurred.");
								}
								g_object_unref(loader);
								return -1;
							}

							GdkPixbuf *buf2 = gdk_pixbuf_loader_get_pixbuf(loader);
							g_object_ref(buf2); //should be un-neccesary, will most likely increase buf2's ref count 1 more than expected
												//and mean it wont free at the end
							g_object_unref(loader);

							tex->truewidth = gdk_pixbuf_get_width(buf2);
							tex->trueheight = gdk_pixbuf_get_height(buf2);
							tex->aspectratio =
							    ((float)tex->truewidth) / ((float)tex->trueheight);

							buf3 =
							    gdk_pixbuf_scale_simple(buf2, 256, 256, GDK_INTERP_TILES);
							    g_assert(buf3);
							buf = gdk_pixbuf_flip(buf3, FALSE);
							tex->width = 256;
							tex->height = 256;
							tex->truewidth = gdk_pixbuf_get_width(buf2);
							tex->trueheight = gdk_pixbuf_get_height(buf2);
							tex->channels = (GLuint) gdk_pixbuf_get_n_channels(buf);
							tex->bpp =
							    (GLuint) gdk_pixbuf_get_bits_per_sample(buf) *
							    tex->channels;

							printf("channels: %d, %d\n", gdk_pixbuf_get_n_channels(buf),
							       gdk_pixbuf_get_bits_per_sample(buf));

							tex->imageData = (GLubyte *) gdk_pixbuf_get_pixels(buf);

							createWindow(win, tex, "", 0, "", 0, "", FALSE);
							g_object_unref(buf3);
							buf3=NULL;
							g_object_unref(buf2);
							buf2=NULL;
							g_object_unref(buf);
							buf=NULL;
						} else
						{

							tex = mem_calloc(sizeof(Texture));
							if (LoadTGATexture(tex, pimage) == 0)
							{
								createWindow(win, tex, "", 0, "", 0, "", FALSE);
							} else
							{
								mem_free(tex);
								errormsg
								    ("An error occurred: Invalid image stored for the member\n");;
							}
						}
					} else
					{
						//nothing to do in this case...
						errormsg("An error occurred: No image located to display\n");
					}
				}
			} else
			{	/*operating mode == 0 */
				pimage = PQgetvalue(res, n, 9);
				message = PQgetvalue(res, n, 5);
				message2 = PQgetvalue(res, n, 10);
				colour = strtol(PQgetvalue(res, n, 11), NULL, 16);
				colour2 = strtol(PQgetvalue(res, n, 12), NULL, 16);
				alerts = PQgetvalue(res, n, 6);
				printf("Sound effect is called next.\n");
				soundeffect(*(PQgetvalue(res, n, 13)));
				id = atoi(PQgetvalue(res, n, 0));
				if (pimage && *pimage)
				{
					int span = strspn(pimage, "0123456789");
					GdkPixbuf *buf,*buf2=NULL,*buf3;
					if (span && pimage[span] == '|' && atoi(pimage) == 0)
					{	// It's an image from the database 
						gsize dlen;
						guint8 *stream = (guint8 *) g_base64_decode(pimage + span + 1, &dlen);
						buf2 =
						    gdk_pixbuf_new_from_inline(dlen, stream, TRUE, &error);

						if (error)
							errormsg("An error occurred: %s\n", error->message);
						g_free(stream);

					} else if (span && pimage[span] == '|' && atoi(pimage) == 1)
					{	// It's a jpeg image from the database 
						gsize dlen;
						guint8 *stream;
						gboolean success;
						GdkPixbufLoader *loader =
							gdk_pixbuf_loader_new_with_type("jpeg", &error);
						if (error)
							errormsg("An error occurred: %s\n", error->message);

						stream = (guint8 *) g_base64_decode(pimage + span + 1, &dlen);
						success = gdk_pixbuf_loader_write(loader, stream, dlen, &error);
						g_free(stream);	/* CHECKED: this should be g _ f r e e */
						if (success)
							success = gdk_pixbuf_loader_close(loader, &error);
						if (!success)
						{
							if (error)
							{
								errormsg("An error occurred: %s\n", error->message);
							} else
							{
								errormsg("An unknown error occurred.");
							}
							buf2=NULL;
						}
						else
						{
							buf2 = gdk_pixbuf_loader_get_pixbuf(loader);
							g_object_ref(buf2);		//shouldn't be required, increases ref count one more than expected, and probably wont be freed
						}
						g_object_unref(loader);
					} else
					{ // it's an image from a file 
						buf2=gdk_pixbuf_new_from_file(pimage, &error);
						if (error)
							errormsg("An error occurred: %s\n", error->message);
					}

					if(buf2)
					 	tex = mem_calloc(sizeof(Texture));
					else
						tex=NULL;
					if(tex)
					{
						tex->aspectratio =
							 ((float)gdk_pixbuf_get_width(buf2)) /
							 ((float)gdk_pixbuf_get_height(buf2));

						buf3 = gdk_pixbuf_scale_simple(buf2, 256, 256, GDK_INTERP_TILES);
						g_assert(buf3);
						buf = gdk_pixbuf_flip(buf3, FALSE);
						tex->width = 256;
						tex->height = 256;
						tex->truewidth = gdk_pixbuf_get_width(buf2);
						tex->trueheight = gdk_pixbuf_get_height(buf2);
						tex->channels = (GLuint) gdk_pixbuf_get_n_channels(buf);
						tex->bpp = (GLuint) gdk_pixbuf_get_bits_per_sample(buf) * tex->channels;
						tex->imageData = (GLubyte *) gdk_pixbuf_get_pixels(buf);
						if(buf3) g_object_unref(buf3);
						buf3=NULL;
						g_object_unref(buf2);
						buf2=NULL;
					}

					createWindow(win, tex, message, colour, message2, colour2, alerts, TRUE);

					if(tex) g_object_unref(buf);

				} else
				{
					//printf("%s, %x, %s, %x, %s\n", (message==NULL?message:"No name"), colour, (message2==NULL?message2:"No name"), colour2, (alerts==NULL?alerts:"No name"));
					createWindow(win, NULL, message, colour, message2,
						     colour2, (alerts ? alerts : ""), TRUE);
				}
			}
			printf("id=%d m%s\n", id, message);

			if (id > lastid)
				lastid = id;

		}
	} else
	{
		for (n = 0; n < rows; n++)
		{
			id = atoi(PQgetvalue(res, n, 0));
			if (id > lastid)
				lastid = id;
		}
	}

	return 1;		/* success */
}

	//select * from swipe(9999,'Mu046b2fc861f40280',false);select * from swipe(9999,'Mu046b2fc861f40280',false);

/* return -1 on error, 0 on nothing happened, and +1 on something happened */
int waitfornotify(int msecs)
{
	extern volatile int server_running;
	SOCKET sock;
	PGresult *res;
	PGnotify *notify;
	fd_set input_mask;
	//printf("WFN\n");
	struct timeval timeout;
	timeout.tv_sec = msecs / 1000;
	timeout.tv_usec = msecs % 1000 * 1000;
	//printf("%d\n", msecs);

	if (!conn)
		reConnectDB();
	sock = (SOCKET) PQsocket(conn);
	if (sock == INVALID_SOCKET)
	{
		reConnectDB();
		sock = (SOCKET) PQsocket(conn);
		if (sock == INVALID_SOCKET)
			return -1;
	}

	if (server_running == 0)
		return -1;
	FD_ZERO(&input_mask);
	FD_SET(sock, &input_mask);

	notify = PQnotifies(conn);
	if (!notify && !freshdb)
	{
		if (select(sock + 1, &input_mask, NULL, NULL, &timeout) < 0)
		{
			fprintf(stderr, "select() failed: %s\n", strerror(errno));
			reConnectDB();
		}

		/* Now check for input */
		if (FD_ISSET(sock, &input_mask))
			PQconsumeInput(conn);
	}
	while (notify || (notify = PQnotifies(conn)) || freshdb)
	{
		char bufr[1024];
		int update;
		freshdb = 0;
		if (notify)
		{
			fprintf(stderr, "ASYNC NOTIFY of '%s' received from backend pid %d\n",
				notify->relname, notify->be_pid);

			if (!strcmp(notify->relname, "shutdown"))
				server_running = 0;

			PQfreemem(notify);
		}
		notify = NULL;

		if (server_running)
		{
			static char *querytext = NULL;

			if (!querytext)
			{
				querytext = bcconfig_getvalue("popup_query");
				if (!querytext)
					querytext = "SELECT doormsg.id,doormsg.doorid,doormsg.membershipid,"
					    "doormsg.memberid,doormsg.date,doormsg.message,doormsg.alerts,"
					    "doormsg.state,door.name,member.image,doormsg.message2,doormsg.colour,doormsg.colour2,doormsg.soundid,doormsg.image "
					    "FROM doormsg "
					    "LEFT OUTER JOIN door "
					    " ON doormsg.doorid=door.id	"
					    "LEFT OUTER JOIN member"
					    " ON doormsg.memberid=member.id " " where doormsg.id > %d";
			}

			sprintf(bufr, querytext, lastid);

			// puts(bufr);
			res = PQexec(conn, bufr);
			printf("Q=\n---\n%s\n---\n", bufr);
			printf("RES=%d\n", PQresultStatus(res));
			fflush(NULL);

			if (PQresultStatus(res) == PGRES_COMMAND_OK || PQresultStatus(res) == PGRES_TUPLES_OK)
			{	// success
				//                      printf("***3\n");fflush(NULL);
				update = updatelast(res);
			}
			//            printf("KX4\n");fflush(NULL);

			PQclear(res);
		}
		return update;
	}
	return 0;
}
